﻿/*
 *Copyright(C) 2023 by  Chief All rights reserved.
 *Unity版本：2022.3.10f1 
 *作者:程一峰  
 *创建日期: 2023-10-20 
 *模块说明：二进制读写辅助
 *版本: 1.0
*/

using System;
using System.IO;
using UnityEngine;
using UnityEngine.Rendering;

namespace Oroxylum
{
    /// <summary>
    /// 通用二进制读写辅助
    /// </summary>
    public static class BinaryHelper
    {

        public static void Write(this BinaryWriter writer, Vector2 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
        }

        public static Vector2 ReadVector2(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            return new Vector2(x, y);
        }

        public static void Write(this BinaryWriter writer, Vector3 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
        }

        public static Vector3 ReadVector3(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            float z = reader.ReadSingle();
            return new Vector3(x, y, z);
        }

        public static void Write(this BinaryWriter writer, Vector4 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
            writer.Write(value.w);
        }

        public static Vector4 ReadVector4(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            float z = reader.ReadSingle();
            float w = reader.ReadSingle();
            return new Vector4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, Quaternion value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
            writer.Write(value.w);
        }

        public static Quaternion ReadUnityQuaternion(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            float z = reader.ReadSingle();
            float w = reader.ReadSingle();
            return new Quaternion(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, Vector2Int value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
        }

        public static Vector2Int ReadVector2Int(this BinaryReader reader)
        {
            int x = reader.ReadInt32();
            int y = reader.ReadInt32();
            return new Vector2Int(x, y);
        }

        public static void Write(this BinaryWriter writer, Vector3Int value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
        }

        public static Vector3Int ReadVector3Int(this BinaryReader reader)
        {
            int x = reader.ReadInt32();
            int y = reader.ReadInt32();
            int z = reader.ReadInt32();
            return new Vector3Int(x, y, z);
        }

        public static void Write(this BinaryWriter writer, Matrix4x4 value)
        {
            writer.Write(value.GetColumn(0));
            writer.Write(value.GetColumn(1));
            writer.Write(value.GetColumn(2));
            writer.Write(value.GetColumn(3));
        }

        public static Matrix4x4 ReadMatrix4x4(this BinaryReader reader)
        {
            Vector4 c0 = reader.ReadVector4();
            Vector4 c1 = reader.ReadVector4();
            Vector4 c2 = reader.ReadVector4();
            Vector4 c3 = reader.ReadVector4();
            return new Matrix4x4(c0, c1, c2, c3);
        }

        public static void Write(this BinaryWriter writer, Matrix4x4[] value)
        {
            if (value == null)
            {
                writer.Write(0);
                return;
            }

            int length = value.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
            {
                var martrix = value[i];
                writer.Write(martrix);
            }
        }

        public static Matrix4x4[] ReadMartix4x4Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Matrix4x4[] array = new Matrix4x4[length];
            for (int i = 0; i < length; i++)
            {
                var martrix = reader.ReadMatrix4x4();
                array[i] = martrix;
            }
            return array;
        }

        public static ShadowCastingMode ReadShadowCastingMode(this BinaryReader reader)
        {
            int mode = reader.ReadInt32();
            return (ShadowCastingMode)mode;
        }

        #region 特殊：为了解决浮点数精度问题

        /*
         * 一般为了解决浮点数精度问题使用这个方法，适用于极小数值
         * 操作方式是将其放大一定比例，然后四舍五入作为 Int 保存
         * 所以适合 0~1 的浮点数，类似旋转信息的保存适合使用这个
         * 
         * ——程一峰；2023.05.08
        */

        public const float FloatToIntRatio = 10000.0f;

        public static void WriteSafe(this BinaryWriter writer, float value)
        {
            int ret = (int)Math.Round(value * FloatToIntRatio);
            writer.Write(ret);
        }

        public static float ReadFloatSafe(this BinaryReader reader)
        {
            int v = reader.ReadInt32();
            return v / FloatToIntRatio;
        }

        public static void WriteSafe(this BinaryWriter writer, Quaternion quaternion)
        {
            writer.WriteSafe(quaternion.x);
            writer.WriteSafe(quaternion.y);
            writer.WriteSafe(quaternion.z);
            writer.WriteSafe(quaternion.w);
        }

        public static Quaternion ReadUnityQuaternionSafe(this BinaryReader reader)
        {
            float x = reader.ReadFloatSafe();
            float y = reader.ReadFloatSafe();
            float z = reader.ReadFloatSafe();
            float w = reader.ReadFloatSafe();
            return new Quaternion(x, y, z, w);
        }

        #endregion

        public static void WriteSafe(this BinaryWriter writer, string data)
        {
            if (data == null)
                data = "";
            writer.Write(data);
        }

        public static void Write(this BinaryWriter writer, string[] strs)
        {
            if (strs == null)
            {
                writer.Write(0);
                return;
            }

            int length = strs.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.WriteSafe(strs[i]);
        }

        public static string[] ReadStringArray(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            string[] strs = new string[length];
            for (int i = 0; i < length; i++)
                strs[i] = reader.ReadString();
            return strs;
        }

        public static void Write(this BinaryWriter writer, int[] array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static int[] ReadIntArray(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            int[] array = new int[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadInt32();
            return array;
        }

        public static void Write(this BinaryWriter writer, Vector2[] array)
        {
            if (array == null)
                writer.Write(0);

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static Vector2[] ReadVector2Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Vector2[] array = new Vector2[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadVector2();
            return array;
        }

        public static void Write(this BinaryWriter writer, Vector3[] array)
        {
            if (array == null)
                writer.Write(0);

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static Vector3[] ReadVector3Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Vector3[] array = new Vector3[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadVector3();
            return array;
        }

        public static void Write(this BinaryWriter writer, Vector4[] array)
        {
            if (array == null)
                writer.Write(0);

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static Vector4[] ReadVector4Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Vector4[] array = new Vector4[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadVector4();
            return array;
        }

        public static void Write(this BinaryWriter writer, Vector3Int[] array)
        {
            if (array == null)
                writer.Write(0);

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static Vector3Int[] ReadVector3IntArray(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Vector3Int[] array = new Vector3Int[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadVector3Int();
            return array;
        }

        public static void Write(this BinaryWriter writer, Vector2Int[] array)
        {
            if (array == null)
                writer.Write(0);

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static Vector2Int[] ReadVector2IntArray(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Vector2Int[] array = new Vector2Int[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadVector2Int();
            return array;
        }

        public static void Write(this BinaryWriter writer, Quaternion[] array)
        {
            if (array == null)
                writer.Write(0);

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static Quaternion[] ReadQuaternionArray(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            Quaternion[] array = new Quaternion[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadUnityQuaternion();
            return array;
        }

    }
}