﻿using System;
using UnityEngine;

namespace Oroxylum
{

    /*
     * 本身Log是非常消耗性能的，但是Log本身也有必要性。
     * 使用这个类可以在不同的情况下对打印进行不同的控制；
     * 用这个打印可以统一性的开启或者关闭打印；
     * 
     * 缺点是不能双击Console窗口跳转到对应代码行；
     */

    /// <summary>
    /// 打印Log的辅助类；
    /// 默认的Log只会在编辑器下生效；
    /// </summary>
    public class Logger
    {


        /// <summary>
        /// 这个Log只会在编辑器下生效；
        /// </summary>
        /// <param name="message"></param>
        public static void Log(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Debug.Log($"{message}");
#endif
        }


        /// <summary>
        /// 普通的Log，以红颜色打印；
        /// </summary>
        /// <param name="message"></param>
        public static void Log_Red(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Log($"<color=red>{message}</color>");
#endif
        }

        /// <summary>
        /// 普通的Log，以红颜色打印；
        /// </summary>
        /// <param name="message"></param>
        public static void Log_Green(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Log($"<color=green>{message}</color>");
#endif
        }

        public static void Log_Blue(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Log($"<color=#0080FF>{message}</color>");
#endif
        }

        public static void Log_Yellow(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Log($"<color=yellow>{message}</color>");
#endif
        }

        public static void Log_Orange(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Log($"<color=FF8000>{message}</color>");
#endif
        }

        /// <summary>
        /// 错误打印;
        /// 这个在真机上不打印；
        /// 除非处于 DEBUG 情况下；
        /// </summary>
        /// <param name="message"></param>
        public static void LogError(object message)
        {
            Debug.LogError($"{message}");
        }

        public static void LogException(Exception ex)
        {
            Debug.LogException(ex);
        }

        public static void LogWarning(object message)
        {
#if UNITY_EDITOR && !RELEASE
            Debug.LogWarning($"{message}");
#endif
        }


        // 修复双击Console窗口无法跳转到对应代码行
#if UNITY_EDITOR

        [UnityEditor.Callbacks.OnOpenAssetAttribute(0)]
        static bool OnOpenAsset(int instanceID, int line)
        {
            string stackTrace = GetStackTrace();
            if (!string.IsNullOrEmpty(stackTrace) && stackTrace.Contains("Logger:Log"))
            {
                System.Text.RegularExpressions.Match matches = System.Text.RegularExpressions.Regex.Match(stackTrace, @"\(at (.+)\)", System.Text.RegularExpressions.RegexOptions.IgnoreCase);
                string pathLine = "";

                //修复点击调用栈的对应行也跳转到调用位置的问题
                var openFileName = UnityEditor.EditorUtility.InstanceIDToObject(instanceID).name;
                if (openFileName != "Logger")
                {
                    return false;
                }

                while (matches.Success)
                {
                    pathLine = matches.Groups[1].Value;

                    if (!pathLine.Contains("Logger.cs"))
                    {
                        int splitIndex = pathLine.LastIndexOf(":");
                        string path = pathLine.Substring(0, splitIndex);
                        line = System.Convert.ToInt32(pathLine.Substring(splitIndex + 1));
                        string fullPath = UnityEngine.Application.dataPath.Substring(0, UnityEngine.Application.dataPath.LastIndexOf("Assets"));
                        fullPath = fullPath + path;
                        UnityEditorInternal.InternalEditorUtility.OpenFileAtLineExternal(fullPath.Replace('/', '\\'), line);
                        break;
                    }
                    matches = matches.NextMatch();
                }
                return true;
            }
            return false;
        }

        static string GetStackTrace()
        {
            var ConsoleWindowType = typeof(UnityEditor.EditorWindow).Assembly.GetType("UnityEditor.ConsoleWindow");
            var fieldInfo = ConsoleWindowType.GetField("ms_ConsoleWindow", System.Reflection.BindingFlags.Static | System.Reflection.BindingFlags.NonPublic);
            var consoleInstance = fieldInfo.GetValue(null);
            if (consoleInstance != null)
            {
                if ((object)UnityEditor.EditorWindow.focusedWindow == consoleInstance)
                {
                    /*
                                        var ListViewStateType = typeof(UnityEditor.EditorWindow).Assembly.GetType("UnityEditor.ListViewState");
                                        fieldInfo = ConsoleWindowType.GetField("m_ListView", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
                                        var listView = fieldInfo.GetValue(consoleInstance);
                                        fieldInfo = ListViewStateType.GetField("row", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.Public);
                                        int row = (int)fieldInfo.GetValue(listView);*/
                    fieldInfo = ConsoleWindowType.GetField("m_ActiveText", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);
                    string activeText = fieldInfo.GetValue(consoleInstance).ToString();
                    return activeText;
                }
            }
            return null;
        }

#endif
    }
}