﻿/*
 *Copyright(C) 2020 by  Cyf All rights reserved.
 *Unity版本：2021.3.11f1c2 
 *作者:程一峰  
 *创建日期: 2022-11-07 
 *模块说明：木蝴蝶数学库
 *版本: 1.0
*/

using System;
using Unity.Mathematics;
using UnityEngine;

namespace Oroxylum
{
    /// <summary>
    /// 通用数学库方法；
    /// </summary>
    public static class OMath
    {
        #region 共用常量

        public const float ZeroTolerance = 1e-08f;

        public static readonly float2 AxisX_2d = new float2(1, 0);
        public static readonly float2 AxisY_2d = new float2(0, 1);
        public static readonly float3 AxisX_3d = new float3(1.0f, 0.0f, 0.0f);
        public static readonly float3 AxisY_3d = new float3(0.0f, 1.0f, 0.0f);
        public static readonly float3 AxisZ_3d = new float3(0.0f, 0.0f, 1.0f);

        #endregion

        #region 值转换

        /// <summary>
        /// 将 float4x4 转换为 Matrix4x4；
        /// </summary>
        /// <param name="value">float4x4</param>
        /// <returns>Matrix4x4</returns>
        public static Matrix4x4 ToMatrix4X4(this float4x4 value) { return new Matrix4x4(value.c0, value.c1, value.c2, value.c3); }

        /// <summary>
        /// 将 Matrix4x4 转换为 float4x4
        /// </summary>
        /// <param name="value">Matrix4x4</param>
        /// <returns>float4x4</returns>
        public static float4x4 ToFloat4X4(this Matrix4x4 value) { return new float4x4(value[0], value[1], value[2], value[3]); }

        public static Vector2 XZ(this Vector3 v) { return new Vector2(v.x, v.z); }

        public static float2 XZ_float2(this Vector3 v) { return new float2(v.x, v.z); }

        #endregion

        #region 值处理

        public static Vector2 Clamp(this Vector2 source, Vector2 min, Vector2 max)
        {
            source.x = math.clamp(source.x, min.x, max.x);
            source.y = math.clamp(source.y, min.y, max.y);
            return source;
        }

        /// <summary>
        /// 距离的平方；
        /// </summary>
        public static float DistanceSq(Vector3 a, Vector3 b)
        {
            float xSq = (a.x - b.x) * (a.x - b.x);
            float ySq = (a.y - b.y) * (a.y - b.y);
            float zSq = (a.z - b.z) * (a.z - b.z);
            return xSq + ySq + zSq;
        }

        /// <summary>
        /// 距离的平方；
        /// </summary>
        public static float DistanceSq(Vector2 a, Vector2 b)
        {
            float xSq = (a.x - b.x) * (a.x - b.x);
            float ySq = (a.y - b.y) * (a.y - b.y);
            return xSq + ySq;
        }

        public static float Dot(this float2 a, float2 b) { return math.dot(a, b); }

        public static float Dot(this float3 a, float3 b) { return math.dot(a, b); }

        public static float3 Cross(this float3 a, float3 b) { return math.cross(a, b); }

        public static float3 Cross(this float2 a, float2 b) { return (a.x * b.y) - (a.y * b.x); }

        public static float3 UnitCross(this float3 v1, float3 v2)
        {
            float3 n = new float3(
                (v1.y * v2.z) - (v1.z * v2.y),
                (v1.z * v2.x) - (v1.x * v2.z),
                (v1.x * v2.y) - (v1.y * v2.x));
            return math.normalize(n);
        }

        public static float LengthSquared(this float2 a) { return math.lengthsq(a); }

        public static float LengthSquared(this float3 a) { return math.lengthsq(a); }

        public static float DotPerp(this float2 a, float2 b) { return (a.x * b.y) - (a.y * b.x); }

        /// <summary>
        /// 垂直向量，向右旋转90度
        /// </summary>
        public static float2 Perp(this float2 a) { return new float2(a.y, -a.x); }

        public static bool IsNormalized(this float2 v) { return Math.Abs((v.x * v.x) + (v.y * v.y) - 1) < ZeroTolerance; }


        #endregion
    }
}